<?php
/**
 * Plumrocket Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End-user License Agreement
 * that is available through the world-wide-web at this URL:
 * http://wiki.plumrocket.net/wiki/EULA
 * If you are unable to obtain it through the world-wide-web, please
 * send an email to support@plumrocket.com so we can send you a copy immediately.
 *
 * @package     Plumrocket_Amp
 * @copyright   Copyright (c) 2017 Plumrocket Inc. (http://www.plumrocket.com)
 * @license     http://wiki.plumrocket.net/wiki/EULA  End-user License Agreement
 *
 * Listener script for AMP IFrame requests
 */

/**
 * For Magento version 1.9.x
 */

// HTTP authentication (Basic access authentication)
const PL_HTTP_AUTH_LOGIN = '';
const PL_HTTP_AUTH_PASS = '';

set_time_limit(0);
chdir(dirname(__FILE__));
error_reporting(E_ALL);
ini_set('display_errors', 1);

// init Mage from Magento
if (!file_exists('app/Mage.php')) {
    echo 'File app/Mage.php not exist';
    exit();
}
require_once 'app/Mage.php';
Mage::app();

// get Data Base config
$params = Mage::getConfig()->getNode('global/resources/default_setup/connection')->asArray();

class AmpImagesSizing
{
    /**
     * @var null|PDO
     */
    protected $pdo = null;
    /**
     * @var null|bool
     */
    protected $isCli = null;
    /**
     * @var string
     */
    protected $tablePrefix = '';
    /**
      * @var array
      */
    protected $tables = array();
    /**
      * @var array
      */
    protected $columns = array();
    /**
     * @var null|Mage_Widget_Model_Template_Filter
     */
    protected $processor = null;
    /**
     * @var string
     */
    protected $auth = '';

    /**
     * Config db tables and they fields
     *
     * @var array[]
     */
    protected $config = array(
        'product' => array(
            'name'  => 'Product',
            'table' => 'catalog_product_entity_text',
            'id'    => 'value_id',
            'value' => 'value',
            'mage'  => true,
        ),
        'category' => array(
            'name'  => 'Categorie',
            'table' => 'catalog_category_entity_text',
            'id'    => 'value_id',
            'value' => 'value',
            'mage'  => true,
        ),
        'cms_block' => array(
            'name'  => 'CMS Block',
            'table' => 'cms_block',
            'id'    => 'block_id',
            'value' => 'content',
            'mage'  => true,
        ),
        'cms_page' => array(
            'name'  => 'CMS Page',
            'table' => 'cms_page',
            'id'    => 'page_id',
            'value' => 'content',
            'mage'  => true,
        ),
        'size_chart' => array(
            'name'  => 'Plumrocket SizeChart',
            'table' => 'pl_size_chart',
            'id'    => 'id',
            'value' => 'content',
            'mage'  => false,
        ),
    );

    /**
     * @var integer
     */
    protected $step = 1000;
        /**
     * @var integer
     */
    protected $count = 0;
    /**
     * @var integer
     */
    protected $success = 0;
    /**
     * @var integer
     */
    protected $already = 0;
    /**
     * @var integer
     */
    protected $error = 0;
    /**
     * @var array
     */
    protected $countByType = array();
    /**
     * @var string
     */
    protected $currentType = '';

    /**
     * AmpImagesSizing constructor.
     *
     * @param string $host
     * @param string $db
     * @param string $user
     * @param string $pass
     */
    public function __construct($host = "", $db = "", $user = "", $pass = "")
    {
        $dsn = "mysql:host=$host;dbname=$db;charset=utf8";
        $opt = array(
            PDO::ATTR_ERRMODE            => PDO::ERRMODE_EXCEPTION,
            PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC
        );
        $this->pdo   = new PDO($dsn, $user, $pass, $opt);
        $this->tablePrefix = (string)Mage::getConfig()->getTablePrefix();
        foreach ($this->config as $type => $config) {
            $this->config[$type]['table'] = $this->tablePrefix . $config['table'];
        }

        /* @var $helper Mage_Cms_Helper_Data */
        $helper = Mage::helper('cms');
        $this->processor = $helper->getBlockTemplateProcessor();

        if (PL_HTTP_AUTH_LOGIN && PL_HTTP_AUTH_PASS) {
            $this->auth = PL_HTTP_AUTH_LOGIN . ':' . PL_HTTP_AUTH_PASS . '@';
        }
        $this->checkCli();
    }

    public function checkCli()
    {
        if ($this->isCli !== null) {
            return $this->isCli;
        }

        if (defined('STDIN')
            || php_sapi_name() === 'cli'
            || array_key_exists('SHELL', $_ENV)
            || empty($_SERVER['REMOTE_ADDR']) and !isset($_SERVER['HTTP_USER_AGENT']) and count($_SERVER['argv']) > 0
            || !array_key_exists('REQUEST_METHOD', $_SERVER)
        ) {
            return $this->isCli = true;
        }
        return $this->isCli = false;
    }

    /**
     * Start set image size
     */
    public function run()
    {
        foreach ($this->config as $type => $config) {
            $this->currentType = $type;
            $this->countByType[$type] = array('count' => 0, 'success' => 0, 'already' => 0, 'error' => 0);
            $this->myEcho(
                'Start set amp size to images in ' . $config['name'] . 's',
                '<div class="section-title">[text]</div>',
                true,
                true
            );
            $isValid = $this->checkTable($config);
            if (!$isValid['error']) {
                $iteration = 0;
                do {
                    // $this->myEcho(false, '<div class="process">');
                    $descriptions = $this->getItems($config, $iteration);
                    $iteration++;
                    foreach ($descriptions as $item) {
                        $newText = $this->getTextWithAmp($item['value']);
                        if ($newText) {
                            $this->updateItems($config, array($item['id'], $newText));
                        }
                    }
                    // $this->myEcho(false, '</div>');
                } while (count($descriptions) == $this->step);
            } else {
                if ($config['mage']) {
                    $this->myEcho($isValid['message'], '<span class="error">[text]</span>', true, true);
                } else {
                    $this->myEcho($isValid['message'], '[text]', true, true);
                }
            }
            $this->myEcho(PHP_EOL, '<br/>');
            $this->showShortStatistic();
            $this->myEcho(PHP_EOL . '------' . PHP_EOL, '<hr/><br/>', false, true);
        }
        $this->showStatistic();
    }

    /**
     * @param array $config
     * @return array
     */
    protected function checkTable($config)
    {
        if (!$this->tables) {
            $stm = $this->pdo->query('SHOW TABLES');
            $stm->execute();
            foreach ($stm->fetchAll() as $array) {
              $this->tables[] = reset($array);
            }
        }

        if (!in_array($config['table'], $this->tables)) {
            return array('error' => true, 'message' => 'Table `' . $config['table'] . '` not exists!');
        }

        if (!isset($this->columns[$config['table']])) {
            $stm = $this->pdo->query('SHOW COLUMNS FROM ' . $config['table']);
            $stm->execute();
            foreach ($stm->fetchAll() as $array) {
                if (isset($array['Field'])) {
                    switch (true) {
                        case $array['Field'] == $config['id']:
                            $this->columns[$config['table']]['id'] = true;
                            break;
                        case $array['Field'] == $config['value']:
                            $this->columns[$config['table']]['value'] = true;
                            break;
                    }
                }
            }
        }

        if (count($this->columns[$config['table']]) !== 2) {
            switch (false) {
                case isset($this->columns[$config['table']]['id']):
                    return array(
                        'error' => true,
                        'message' => 'Column `' . $config['table'] . '`.`' . $config['id'] . '` not exists!',
                    );
                case isset($this->columns[$config['table']]['value']):
                    return array(
                        'error' => true,
                        'message' => 'Column `' . $config['table'] . '`.`' . $config['value'] . '` not exists!',
                    );
            }
        }

        return array('error' => false);
    }

    /**
     * @param array $config
     * @param int   $iteration
     * @return array
     */
    protected function getItems($config, $iteration)
    {
        $stm = $this->pdo->query('
                SELECT
                  `' . $config['id'] . '` as id,
                  `' . $config['value'] . '` as value
                FROM  `' . $config['table'] . '`
                WHERE `' . $config['value'] . '` IS NOT NULL
                ORDER BY `' . $config['id'] . '`
                LIMIT ' . $this->step  . '
                OFFSET ' . $iteration * $this->step  . '
            ');
        $stm->execute();
        return $stm->fetchAll();
    }

    /**
     * @param array $config
     * @param array $item
     */
    protected function updateItems($config, $item)
    {
        list($id, $value) = $item;
        $stmt = $this->pdo->prepare('
            UPDATE ' . $config['table'] . '
            SET ' . $config['value'] . ' = :value
            WHERE `' . $config['id'] . '` = :id
        ');
        $stmt->bindParam(':id',    $id,    PDO::PARAM_INT);
        $stmt->bindParam(':value', $value, PDO::PARAM_STR);
        $stmt->execute();
    }

    /**
     * @param string $text
     * @return string|bool
     */
    protected function getTextWithAmp($text)
    {
        if ($text) {
            $source = array();
            preg_match_all('/<img[\s\n\r\S]*?src="[\s\n\r\S]*?\/?>/', $text, $source);
            if ($source) {
                $result = array();
                foreach (array_unique($source[0]) as $img) {
                    $result[] = array(
                        'source' => $img,
                        'new'    => $this->getImageWithAmpParam($img),
                    );
                }
                foreach ($result as $item) {
                    if ($item['new']) {
                        $text = str_replace($item['source'], $item['new'], $text);
                    }
                }
                return $text;
            }
        }
        return false;
    }

    /**
     * @param string $img
     *
     * @return string|bool
     */
    protected function getImageWithAmpParam($img)
    {
        $this->count++;
        $this->countByType[$this->currentType]['count']++;
        $result = false;
        preg_match('/data-width-amp|data-height-amp|layout="/', $img, $result);
        if ($result) {
            $this->already++;
            $this->countByType[$this->currentType]['already']++;
            // $this->myEcho('-', '[text]', true);
            return false;
        }
        $imgPath = '';
        $img = trim(preg_replace('/[\n\r]/', ' ', $img));
        $img = preg_replace('/src=["\s]+(.+?)["\s]+/', 'src="$1"', $img);
        $result = array();
        switch (true) {
            case strpos($img,'src="{{') !== false :
                preg_match('/{{.*?}}/', $img, $result);
                $imgPath = $this->processor->filter($result[0]);
                break;

            case strpos($img,'src="http') !== false :
                preg_match('/src="(.*?)"/', $img, $result);
                if (isset($result[1])) {
                    $imgPath = $result[1];
                }
                break;

            default:
                preg_match('/src="(.*?)"/', $img, $result);
                if (isset($result[1])) {
                    $imgPath = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_MEDIA)
                             . ((strpos($result[1], '/') === 0) ? substr($result[1], 1) : $result[1]);
                }
                break;
        }

        return $this->setAmpParam($img, $this->getAmpParam(trim($imgPath)));
    }

    /**
     * @param string $imgPath
     *
     * @return array
     */
    protected function getAmpParam($imgPath)
    {
        if ($imgPath && strpos($imgPath, 'http') === 0) {
            if ($this->auth) {
                $imgPath = preg_replace('/(^https?:\/\/)/', '$1' . $this->auth, $imgPath);
            }

            $imgPath = str_replace(' ', '', $imgPath);

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $imgPath);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_NOBODY, 0);
            $file_contents = curl_exec($ch);
            curl_close($ch);

            $data = ImageCreateFromString($file_contents);
            $imageSize = array(ImageSX($data), ImageSY($data));

            if (isset($imageSize[0], $imageSize[1]) && $imageSize[0] && $imageSize[1]) {
                return array('width' => $imageSize[0], 'height' => $imageSize[1]);
            } else {
                /**
                 * Alternative for curl
                 */
                $imageSize = getimagesize($imgPath);
                if (isset($imageSize[0], $imageSize[1]) && $imageSize[0] && $imageSize[1]) {
                    return array('width' => $imageSize[0], 'height' => $imageSize[1]);
                }
            }
            $this->error++;
            $this->countByType[$this->currentType]['error']++;
            // $this->myEcho('!', '<span class="error">[text]</span>', true);
            // $this->myEcho(PHP_EOL, false);
            $this->myEcho(' Fail get size: ' . $imgPath,
                '<div class="img-path"><span class="error img-path"> Fail get size: </span>'
                    . '<span class="text">' . $imgPath . '</span></div>'
            );
            $this->myEcho(PHP_EOL, false);
            $this->log('fail get size: ' . $imgPath);
        }
        return array();
    }

    /**
     * @param string $img
     * @param array $param
     *
     * @return string
     */
    protected function setAmpParam($img, $param)
    {
        if ($param) {
            $img = str_replace('<img', '<img' . $this->prepareParams($param), $img);
            $img = preg_replace('/\s+/', ' ', $img);
            $this->success++;
            $this->countByType[$this->currentType]['success']++;
            // $this->myEcho('.', '[text]', true);
        }
        return $img;
    }

    /**
     * Make html attributes from array
     *
     * @param $params
     *
     * @return string
     */
    protected function prepareParams($params)
    {
        $result = ' layout="responsive" ';
        foreach ($params as $attr => $val) {
            $result .= 'data-' . $attr . '-amp="' . $val . '"';
        }
        return $result;
    }

    public function log($message)
    {
        Mage::log($message, null, 'amp-image.log', true);
    }

    public function showStatistic()
    {
        $this->myEcho(
            'All: ' . $this->count . PHP_EOL
            . 'Success: ' . ($this->success ? $this->success : '0')
            . ' (' . round(100 * $this->success / $this->count, 2) . '%)' . PHP_EOL
            . 'Already: ' . ($this->already ? $this->already : '0')
            . ' (' . round(100 * $this->already / $this->count, 2) . '%)' . PHP_EOL
            . 'Error:' . ($this->error ? $this->error : '0')
            . ' (' . round(100 * $this->error / $this->count, 2) . '%)' . PHP_EOL,
            '<table class="statistic">
                <tr>
                    <th>All</th>
                    <td>' . $this->count . '</td><td></td>
                </tr>
                <tr class="success">
                    <th>Success</th>
                    <td>' . ($this->success ? $this->success : '0') . '</td>
                    <td>' . round(100 * $this->success / $this->count, 2) . '%</td>
                </tr>
                <tr>
                    <th>Already</th>
                    <td>' . ($this->already ? $this->already : '0') . '</td>
                    <td>' . round(100 * $this->already / $this->count, 2) . '%</td>
                </tr>
                <tr class="error">
                    <th>Error</th>
                    <td>' . ($this->error ? $this->error : '0') . '</td>
                    <td>' . round(100 * $this->error / $this->count, 2) . '%</td>
                </tr>
            </table>'
        );
    }

    public function showShortStatistic()
    {
        if (isset($this->countByType[$this->currentType])) {
            $statistic = $this->countByType[$this->currentType];
            $this->myEcho('Count: ' . $this->toStr($statistic['count'])
                . ' (Success: ' . $this->toStr($statistic['success'])
                . ' | Already:' . $this->toStr($statistic['already'])
                . ' | Error:' . $this->toStr($statistic['error']). ')',
                '<div class="statistic">All: '
                . $this->toStr($statistic['count'])
                . ' (<span class="success">Success: ' . $this->toStr($statistic['success'])
                . '</span> | Already:' . $this->toStr($statistic['already'])
                . ' | <span class="error">Error:' . $this->toStr($statistic['error']). '</span>)'
                . '</div>'
            );
        }
    }

    public function myEcho($console, $html, $wrapper = false, $br = false)
    {
        if ($this->isCli && $console) {
            echo $console . ($br ? PHP_EOL : '');
        }
        if (!$this->isCli && $html) {
            if ($wrapper) {
                echo str_replace('[text]', $console, $html);
            } else {
                echo $html;
            }
        }
    }

    protected function toStr($int)
    {
        return $int ? $int : '0';
    }
}

$a = new AmpImagesSizing($params['host'],  $params['dbname'],  $params['username'],  $params['password'], $is_cli);

$a->myEcho(false, '<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <title>Plumrocket AMP Image Sizing</title>
  <style>
    @import url(https://fonts.googleapis.com/css?family=Share+Tech+Mono&subset=latin,cyrillic);
    body {
      background-color: #292929;
      padding: 8px;
      margin: 0;
    }
    *, .text {
      color: #f9f9f9;
    }
    .section-title {
      font-family: "Share Tech Mono", monospace;
      font-size: 21px;
    }
    .statistic {
      font-family: "Share Tech Mono", monospace;
      font-size: 18px;
    }
    table {
        border-collapse: collapse;
    }
    .statistic th {
      text-align: right;
    }
    .statistic td, .statistic td {
      padding: 2px 15px;
    }
    .statistic td, .statistic th {
      border-bottom: 1px solid #f9f9f9;
    }
    .statistic tbody tr:last-child td, .statistic tbody tr:last-child th {
      border-bottom: 0;
    }
    .success, .success td, .success th {
      color: #11d023;
    }
    div.process {
        color: #f9f9f9;
        font-size: 20px;
    }
    .error, .error td, .error th {
      color: #ea0e0e;
    }
    .img-path {
        font-size: 14px;
        font-family: arial, monospace;
    }
    div.img-path {
        padding-left: 10px;
    }
</style>
</head>
<body>');

$a->run();

$a->myEcho(false, '</body></html>');